define("jira/automation-configuration-view", [
  "jquery",
  "backbone",
  "underscore",
  "jira/model/automation-service",
  "jira/automation-utils",
  'jira/autocomplete/jql-autocomplete',
  'jira/jql/jql-parser',
  'jira/util/forms'
], function ($,
  Backbone,
  _, AutomationService,
  Utils,
  JQLAutoComplete,
  JQLParser,
  Forms) {
  var ConfigView = Backbone.View.extend({
    el: ".automation-configuration-wrapper",
    events: {
      "click #save-widget-configuration": "saveConfiguration",
      "change .checkbox": "handleCheckboxData",
      "change #select2-triggerType": "handleSelectTriggerType",
      "change #select2-action-template": "handleSelectActionTemplate",
      "click .aui-tabs": "handChangeTab",
      "keyup #expressionCondition": "validateExpression",
      "click .edit-auto-rule": "handEditRule",
      "click .view-log-auto-rule": "handleViewLogs",
      "click .remove-auto-rule": "handRemoveRule",
      "click .down-auto-rule": "handleDownRule",
      "click .up-auto-rule": "handleUpRule",
      "click .show-config-map": "showConfigMapDialog",
      "click .remove-config-map": "removeConfigMap"
    },
    initialize: function (options) {
      this.selectedAction = "";
      this.projectKey = options.projectKey;
      this.render();
    },
    render: function () {
      this.initActionScript();
      this.initWebhookHeader()
      this.loadEvents();
      this.loadActions();
      this.loadVariables();
      this.loadRules();
      this.initJQL();
      this.loadConfigMaps();
    },
    initJQL: function () {
      var self = this;
      AutomationService.jqlautocompletedata().done(function (meta) {
        var jqlFieldNames = meta.visibleFieldNames;
        var jqlFunctionNames = meta.visibleFunctionNames;
        var jqlReservedWords = meta.jqlReservedWords;

        var jqlAutoComplete = JQLAutoComplete({
          fieldID: 'jqltext',
          parser: JQLParser(jqlReservedWords),
          queryDelay: 0.65,
          jqlFieldNames: jqlFieldNames,
          jqlFunctionNames: jqlFunctionNames,
          minQueryLength: 1,
          allowArrowCarousel: true,
          autoSelectFirst: false,
          errorID: 'jqlerrormsg'
        });
      });
    },
    initWebhookHeader: function () {
      var self = this;
      var $actionScript = self.$el.find("#webhookHeader")[0];
      self.webhookHeaderCodeEditor = CodeMirror.fromTextArea($actionScript, {
        mode: "text/x-groovy",
        lineNumbers: true,
        lineWrapping: true
      });
    },
    initActionScript: function () {
      var self = this;
      var $actionScript = self.$el.find("#actionScript")[0];
      self.actionScriptCodeEditor = CodeMirror.fromTextArea($actionScript, {
        mode: "text/x-groovy",
        lineNumbers: true,
        lineWrapping: true
      });
      self.actionScriptCodeEditor.on('change', function (e) {
        self.validateActionScript(e, self.selectedAction);
      });
    },
    loadEvents: function () {
      var self = this;
      AutomationService.getEvents(self.projectKey).then(function (events) {
        _.each(events, function (event) {
          $("#select2-events").append($('<option>', {
            value: event.key,
            text: event.name
          }));
        });
        self.events = events;
        $("#select2-events").auiSelect2();
      });
    },
    loadActions: function () {
      var self = this;
      AutomationService.getActions(self.projectKey).then(function (actions) {
        _.each(actions, function (action) {
          $("#select2-action-template")
            .append($('<option>', {
              value: action.key,
              text: action.name
            }));
        });
        self.actions = actions;
        $("#select2-action-template").auiSelect2();
      });
    },
    loadVariables: function () {
      var self = this;
      AutomationService.getContextVariables(self.projectKey).then(function (contexts) {
        var $html = $(JIRA.Templates.Automation.contextVariables({ contexts: contexts }));
        self.$el.find("#context-variables").html($html);
      });
    },
    loadRules: function () {
      var self = this;
      AutomationService.getRules(self.projectKey).then(function (rules) {
        var $html = $(JIRA.Templates.Automation.rules({ rules: rules }));
        self.$el.find("#rule-list").html($html);
      });
    },
    loadConfigMaps: function () {
      var self = this;
      AutomationService.getConfigMaps(self.projectKey).then(function (configMaps) {
        var $html = $(JIRA.Templates.Automation.configMaps({ configMaps: configMaps }));
        self.$el.find("#config-maps").html($html);
      });
    },
    getFormData2Object: function (form) {
      var un_array = form.serializeArray();
      var _array = {};
      $.map(un_array, function (n, i) {
        if (n.name.indexOf('[') > -1) {
          var array = n.name.match(/\[(.*?)\]/);
          var key = n.name.replace(array[1], "").replace('[', "").replace(']', "");
          if (!_array[key]) {
            _array[key] = {};
          }
          _array[key][array[1]] = n['value'];
        } else {
          _array[n['name']] = n['value'];
        }
      });
      return _array;
    },
    saveConfiguration: function (e) {
      var self = this;
      var ok = self.validateRule();
      if (ok) {
        this.actionScriptCodeEditor.save();
        this.webhookHeaderCodeEditor.save();
        var param = this.getFormData2Object($('#automation-config-form'));
        param.projectKey = this.projectKey;
        param.triggerEvents = $("#select2-events").val().join(',');
        AutomationService.saveProjectConfig(param).done(function (res) {
          Utils.notifySuccessMessage("Save setting!");
          self.loadRules();
        });
      }
    },
    handleCheckboxData: function (e) {
      var data = $(e.target).val();
      data = data === 'true' ? 'false' : 'true';
      $(e.target).val(data);
      e.stopPropagation();
    },
    handChangeTab: function () {
      this.actionScriptCodeEditor.refresh();
      this.webhookHeaderCodeEditor.refresh();
    },
    handleSelectActionTemplate: function (e) {
      var self = this;
      var data = $(e.target).val();
      _.each(self.actions, function (action) {
        if (action.key === data) {
          self.actionScriptCodeEditor.setValue(action.template);
          self.actionScriptCodeEditor.refresh();
          self.selectedAction = data;
        }
      });
    },
    handleSelectTriggerType: function (e) {
      var self = this;
      var selectedTriggerType = $(e.target).val();
      if (selectedTriggerType === 'ISSUE_EVENT') {
        self.$el.find('.cron').addClass('hidden');
        self.$el.find('.webhook').addClass('hidden');
        self.$el.find('.issue-events').removeClass('hidden');
      }
      if (selectedTriggerType === 'WEBHOOK') {
        this.webhookHeaderCodeEditor.refresh();
        self.$el.find('.cron').addClass('hidden');
        self.$el.find('.issue-events').addClass('hidden');
        self.$el.find('.webhook').removeClass('hidden');
      }
      if (selectedTriggerType === 'CRON') {
        self.$el.find('.webhook').addClass('hidden');
        self.$el.find('.issue-events').addClass('hidden');
        self.$el.find('.cron').removeClass('hidden');
      }
    },
    validateRule: function () {
      var self = this;
      var nameValue = self.$el.find('#name').val();
      if (nameValue === '' || nameValue === undefined) {
        Utils.notifyErrorMessage("Name must not be empty");
        return false;
      }
      var selectedTriggerType = self.$el.find('#select2-triggerType').val();
      if (selectedTriggerType === 'ISSUE_EVENT') {
        var eventsValue = self.$el.find('#select2-events').val();
        if (eventsValue === '' || eventsValue === undefined) {
          Utils.notifyErrorMessage("Events must not be empty");
          return false;
        }
      }
      if (selectedTriggerType === 'CRON') {
        var cronValue = self.$el.find('#triggerCron').val();
        if (cronValue === '' || cronValue === undefined) {
          Utils.notifyErrorMessage("Cron expression must not be empty");
          return false;
        }
      }
      if (selectedTriggerType === 'WEBHOOK') {
        var webhookUrl = self.$el.find('#webhookUrl').val();
        if (webhookUrl === '' || webhookUrl === undefined) {
          Utils.notifyErrorMessage("WebhookUrl must be valid");
          return false;
        }
      }

      var jqlValue = self.$el.find('#jql').val();
      var expressionConditionValue = self.$el.find('#expressionCondition').val();
      if (jqlValue === undefined && expressionConditionValue === undefined) {
        Utils.notifyErrorMessage("Condition must be not be empty");
        return false;
      }
      return true;
    },
    handEditRule: function (e) {
      var self = this;
      var ruleId = $(e.target).data('rule-id');
      AutomationService.getConfig(ruleId).done(function (rule) {
        self.actionScriptCodeEditor.setValue(rule.actionScript);
        self.$el.find("#select2-action-template").val(rule.actionTemplate).trigger('change');
        self.$el.find("#enableAuditLog").val(rule.enableAuditLog);
        self.$el.find("#enableRule").val(rule.enableRule);
        self.$el.find("#expressionCondition").val(rule.expressionCondition);
        self.$el.find("#jql").val(rule.jql);
        var events = rule.triggerEvents.split(',');
        self.$el.find("#select2-triggerType").val(rule.triggerType).trigger('change');
        self.$el.find("#select2-events").val(events).trigger('change');
        self.$el.find("#name").val(rule.name);
        self.$el.find("#webhookHeader").val(rule.webhookHeader);
        self.$el.find("#webhookUrl").val(rule.webhookUrl);
        self.$el.find("#enableRule").prop('checked', rule.enableRule);
        self.$el.find("#enableAuditLog").prop('checked', rule.enableAuditLog);
      });
    },
    handRemoveRule: function (e) {
      var self = this;
      var ruleId = $(e.target).data('rule-id');
      var result = confirm("Are you sure?");
      if (result) {
        AutomationService.removeConfig(ruleId).done(function (res) {
          Utils.notifySuccessMessage("Remove rule!");
          self.loadRules();
        });
      }
    },
    handleViewLogs: function (e) {
      var self = this;
      var ruleId = $(e.target).data('rule-id');
      AutomationService.getLogs(ruleId).done(function (logs) {
        var $html = $(JIRA.Templates.Automation.logs({ logs: logs, ruleId: ruleId }));
        self.$el.find("#rule-logs").html($html);
      });
    },
    validateExpression: function (e) {
      var self = this;
      if (event.keyCode == 13) {
        var value = $(e.target).val();
        var type = $(e.target).data('type');
        AutomationService.testPostScript(value, type).done(function (res) {
          self.$el.find(".expression-condition .icon").addClass('jqlgood');
          self.$el.find(".expression-condition .icon").removeClass('jqlerror');
          Utils.notifySuccessMessage("Expression looks good!!!");
        }).error(function (res) {
          self.$el.find(".expression-condition .icon").addClass('jqlerror');
          self.$el.find(".expression-condition .icon").removeClass('jqlgood');
          Utils.notifyErrorMessage("Oh no!!!: " + JSON.parse(res.responseText).message);
        });
      }
    },
    validateActionScript: function (e, selectedAction) {
      var self = this;
      if (event.keyCode == 13) {
        var value = self.actionScriptCodeEditor.getValue();
        if (selectedAction !== "") {
          AutomationService.testPostScript(value, selectedAction).done(function (res) {
            self.$el.find(".action-script .icon").addClass('jqlgood');
            self.$el.find(".action-script .icon").removeClass('jqlerror');
            Utils.notifySuccessMessage("Action script looks good!!!");
          }).error(function (res) {
            self.$el.find(".action-script .icon").addClass('jqlerror');
            self.$el.find(".action-script .icon").removeClass('jqlgood');
            Utils.notifyErrorMessage("Oh no!!!: " + JSON.parse(res.responseText).message);
          });
        } else {
          Utils.notifyErrorMessage("Please select an action template");
        }

      }
    },
    handleUpRule: function(e) {
      var self = this;
      var ruleId = $(e.target).data('rule-id');
      var ruleOrderNum = $(e.target).data('order-num') || 1;
      AutomationService.changeOrderNum(ruleId, ruleOrderNum - 1).done(function (res) {
        Utils.notifySuccessMessage("Up rule!");
        self.loadRules();
      });
    },
    handleDownRule: function(e) {
      var self = this;
      var ruleId = $(e.target).data('rule-id');
      var ruleOrderNum = $(e.target).data('order-num') || 0;
      AutomationService.changeOrderNum(ruleId, ruleOrderNum + 1).done(function (res) {
        Utils.notifySuccessMessage("Down rule!");
        self.loadRules();
      });
    },
    showConfigMapDialog: function(e) {
        var self = this;
        var $dialog = AJS.dialog2("#project-config-dialog").$el;
        $dialog.find(".config-map-save").click(function () {
            self.saveConfigMap();
        });
        $dialog.find(".config-map-close").click(function () {
            AJS.dialog2("#project-config-dialog").hide();
        });
        AJS.dialog2("#project-config-dialog").show();
        var configMap = {
            name: "",
            key: "",
            value: "",
            type: 'TEXT'
        };
        if(e && e.target) {
            var configMapKey = $(e.target).data('key');
            AutomationService.getConfigMap(self.projectKey, configMapKey).done(function(configMapRes) {
                if(configMapRes) {
                    $dialog.find(".aui-dialog2-content").html(JIRA.Templates.Automation.configMapForm({config: configMapRes, editMode: true}));
                } else {
                    configMap.key = configMapKey;
                    $dialog.find(".aui-dialog2-content").html(JIRA.Templates.Automation.configMapForm({config: configMap, editMode: true}));
                }
            });
        } else {
            $dialog.find(".aui-dialog2-content").html(JIRA.Templates.Automation.configMapForm({config: configMap, editMode: false}));
        }

    },
    saveConfigMap: function(e) {
        var self = this;
        var $dialog = AJS.dialog2("#project-config-dialog").$el;
        var configMap = self.getFormData2Object($dialog.find("#configMapForm"));
        configMap.projectKey = self.projectKey;
        if(configMap.editMode) {
            AutomationService.saveConfigMap(configMap).done(function (res) {
                Utils.notifySuccessMessage("Save config map!");
                AJS.dialog2("#project-config-dialog").hide();
                self.loadConfigMaps();
            });
        } else {
            AutomationService.validateConfigMap(self.projectKey, configMap.key).done(function(ok) {
                if(!ok) {
                    AutomationService.saveConfigMap(configMap).done(function (res) {
                        Utils.notifySuccessMessage("Save config map!");
                        AJS.dialog2("#project-config-dialog").hide();
                        self.loadConfigMaps();
                    });
                } else {
                    Utils.notifyErrorMessage("Key is existed in this project");
                }
            });
        }

    },
    removeConfigMap: function(e) {
        var self = this;
        var $dialog = AJS.dialog2("#project-config-dialog").$el;
        var configMapKey = $(e.target).data('key');
        var result = confirm("Are you sure? It may used in action script or condition!!!");
        if (result) {
          AutomationService.removeConfigMap(self.projectKey, configMapKey).done(function (res) {
              Utils.notifySuccessMessage("Delete config map!");
              AJS.dialog2("#project-config-dialog").hide();
              self.loadConfigMaps();
          });
        }
    }
  });
  return ConfigView;
});
